//! Represents a command that can be included in a function.

mod execute;

pub use execute::{Condition, Execute};

use serde::{Deserialize, Serialize};

use crate::util::compile::{CompileOptions, MutCompilerState, MutFunctionCompilerState};

use super::Function;

/// Represents a command that can be included in a function.
#[derive(Debug, Clone, Serialize, Deserialize)]
pub enum Command {
    /// A command that is already formatted as a string.
    Raw(String),
    /// Message to be printed only in debug mode
    Debug(String),
    /// Execute command
    Execute(Execute),
    /// Group of commands to be called instantly after each other
    Group(Vec<Command>),
}

impl Command {
    /// Create a new raw command.
    pub fn raw(command: &str) -> Self {
        Self::Raw(command.to_string())
    }

    /// Compile the command into a string.
    pub fn compile(
        &self,
        options: &CompileOptions,
        global_state: &MutCompilerState,
        function_state: &MutFunctionCompilerState,
    ) -> Vec<String> {
        match self {
            Self::Raw(command) => vec![command.clone()],
            Self::Debug(message) => compile_debug(message, options),
            Self::Execute(ex) => ex.compile(options, global_state, function_state),
            Self::Group(commands) => {
                // TODO: implement correctly
                commands
                    .iter()
                    .flat_map(|c| c.compile(options, global_state, function_state))
                    .collect()
            }
        }
    }
}

impl From<&str> for Command {
    fn from(command: &str) -> Self {
        Self::raw(command)
    }
}
impl From<&Function> for Command {
    fn from(value: &Function) -> Self {
        Self::Raw(format!("function {}:{}", value.namespace(), value.name()))
    }
}
impl From<&mut Function> for Command {
    fn from(value: &mut Function) -> Self {
        Self::Raw(format!("function {}:{}", value.namespace(), value.name()))
    }
}

fn compile_debug(message: &str, option: &CompileOptions) -> Vec<String> {
    if option.debug {
        vec![format!(
            r#"tellraw @a [{{"text":"[","color":"dark_blue"}},{{"text":"DEBUG","color":"dark_green","hoverEvent":{{"action":"show_text","value":[{{"text":"Debug message generated by Shulkerbox"}},{{"text":"\nSet debug message to 'false' to disable"}}]}}}},{{"text":"]","color":"dark_blue"}},{{"text":" {}","color":"black"}}]"#,
            message
        )]
    } else {
        Vec::new()
    }
}
